@@ -68,7 +68,7 @@ def install_dist(distname, workdir):
     scripts = pfx + '--install-scripts=%s/scripts' % workdir
     data = pfx + '--install-data=%s/data' % workdir
     cmd = ['pip', 'install',
-           '--index-url', 'https://pypi.python.org/simple/',
+           '--index-url', 'https://pypi.org/simple/',
            '--timeout', '3', '--default-timeout', '3',
            purelib, platlib, headers, scripts, data, distname]
     result = {
@@ -354,6 +354,10 @@ class WheelTestCase(unittest.TestCase):
         fn = os.path.join(HERE, 'dummy-0.1-py27-none-any.whl')
         w = Wheel(fn)
         w.verify()
+        # see issue 115
+        fn = os.path.join(HERE, 'valid_wheel-0.0.1-py3-none-any.whl')
+        w = Wheel(fn)
+        w.verify()
         fn = os.path.join(HERE, 'bad_wheels', 'dummy-0.1-py27-none-any.whl')
         w = Wheel(fn)
         self.assertRaises(DistlibException, w.verify)
@@ -482,6 +486,8 @@ class WheelTestCase(unittest.TestCase):
 
     @unittest.skipIf('SKIP_ONLINE' in os.environ, 'Skipping online test')
     @unittest.skipUnless(PIP_AVAILABLE, 'pip is needed for this test')
+    @unittest.skipIf(sys.version_info[:2] >= (3, 7), 'The test distribution is not '
+                                               '3.7+ compatible')
     def test_build_and_install_pure(self):
         self.do_build_and_install('sarge == 0.1')
 
